/* doc.h
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#ifndef N96_DRAW_doc_h
#define N96_DRAW_doc_h

#include "floodfill.h"
#include "layer.h"
#include "image-cairo.h"
#define STACKLIMIT (5)
#define DEBUG_COMPOSITE_DIRTYRECT (false)
#define RECOMPOSITE_DIRTY_ONLY (true)
#define REPREVIEW_DIRTY_ONLY (true)
typedef struct HistoryPoint HistoryPoint;
typedef struct _ngg_vtab_t_HistoryPoint {
} _ngg_vtab_t_HistoryPoint;

typedef struct HistoryPoint {
	_ngg_vtab_t_HistoryPoint _ngg_vtab_history_point;
	Image *fgimg;
	Image *bgimg;
} HistoryPoint;

typedef struct HistoryPointStack HistoryPointStack;
typedef struct _ngg_vtab_t_HistoryPointStack {
} _ngg_vtab_t_HistoryPointStack;

typedef struct HistoryPointStack {
	_ngg_vtab_t_HistoryPointStack _ngg_vtab_history_point_stack;
	HistoryPoint* *arr;
	size_t count;
	size_t alloccount;
} HistoryPointStack;

typedef struct ImageDocument ImageDocument;
typedef struct _ngg_vtab_t_ImageDocument {
} _ngg_vtab_t_ImageDocument;

typedef struct ImageDocument {
	_ngg_vtab_t_ImageDocument _ngg_vtab_image_document;
	struct Pixel bgcolor;
	Image *checkpattern;
	Layer *fglayer;
	Layer *bglayer;
	struct Rectangle dirtyrect;
	Image *composite;
	Image *preview;
	Layer *layer;
	_Bool pen_is_down;
	double oldx;
	double oldy;
	double stroke_smoothness;
	_Bool subpixel_when_zoomed;
	struct DrawingParams drawparams;
	double fgalpha;
	_Bool bgvisible;
	int vpx;
	int vpy;
	double vpzoom;
	int oldpreviewx;
	int oldpreviewy;
	int oldprevieww;
	int oldpreviewh;
	double oldpreviewzoom;
	_Bool autocenter;
	HistoryPointStack *undostack;
	HistoryPointStack *redostack;
} ImageDocument;

static inline int image_document_get_height(ImageDocument * this) {
return this->fglayer->img->height;
}
static inline int image_document_get_width(ImageDocument * this) {
return this->fglayer->img->width;
}
typedef struct _ngg_tuple_update_composite {
	_Bool m0;
	struct Rectangle m1;
} _ngg_tuple_update_composite;

Image* history_point_detach_bgimg(HistoryPoint *this);
Image* history_point_detach_fgimg(HistoryPoint *this);
void history_point_construct(HistoryPoint *this);
void history_point_destruct(HistoryPoint *this);
void history_point_stack_construct(HistoryPointStack *this);
void history_point_stack_destruct(HistoryPointStack *this);
_Bool history_point_stack_is_empty(HistoryPointStack *this);
void history_point_stack_push(HistoryPointStack *this, HistoryPoint *newitem);
void history_point_stack_push_multiple(HistoryPointStack *this, size_t n, HistoryPoint* *newitems);
HistoryPoint* history_point_stack_peek(HistoryPointStack *this);
HistoryPoint* history_point_stack_pop(HistoryPointStack *this);
HistoryPoint* history_point_stack_pop_bottom(HistoryPointStack *this);
void history_point_stack_replace_top(HistoryPointStack *this, HistoryPoint *newitem);
void history_point_stack__resize(HistoryPointStack *this, size_t newcount);
struct DrawingParams default_drawparams();
void image_document_construct(ImageDocument *this, int width, int height);
void image_document_destruct(ImageDocument *this);
void image_document_apply_snapshot(ImageDocument *this, HistoryPoint *hp);
_Bool image_document_can_redo(ImageDocument *this);
_Bool image_document_can_undo(ImageDocument *this);
void image_document_clear_layer(ImageDocument *this);
void image_document_clear_redostack(ImageDocument *this);
void image_document_clear_undostack(ImageDocument *this);
HistoryPoint* image_document_create_history_point(ImageDocument *this);
HistoryPoint* image_document_create_redo_snapshot(ImageDocument *this);
void image_document_create_undo_snapshot(ImageDocument *this);
void image_document_dip_eyedropper(ImageDocument *this, int x, int y);
void image_document_export_document(ImageDocument *this, const char * filename, _Bool fillbg);
void image_document_export_layer(ImageDocument *this, const char * filename, _Bool fillbg);
void image_document_fill_layer(ImageDocument *this, struct Pixel color);
void image_document_fill_layer_with_bgcolor(ImageDocument *this);
void image_document_fill_layer_with_fgcolor(ImageDocument *this);
void image_document_floodfill(ImageDocument *this, int x, int y);
struct Pixel image_document_get_color_at(ImageDocument *this, int x, int y);
void image_document_add_dirty_region(ImageDocument *this, struct Rectangle rect);
_Bool image_document_is_all_dirty(ImageDocument *this);
_Bool image_document_is_dirty(ImageDocument *this);
void image_document_mark_all_dirty(ImageDocument *this);
void image_document_mark_not_dirty(ImageDocument *this);
void image_document_pen_down(ImageDocument *this, int x, int y, double pressure);
void image_document_pen_up(ImageDocument *this, int x, int y, double pressure);
void image_document_pen_stroke(ImageDocument *this, int ix, int iy, double pressure);
double image_document_translate_x(ImageDocument *this, double x);
double image_document_translate_y(ImageDocument *this, double y);
void image_document_update_pressure(ImageDocument *this, double pressure);
void image_document_undo(ImageDocument *this);
void image_document_viewport_align_center(ImageDocument *this);
void image_document_viewport_set_offsetx(ImageDocument *this, int x);
void image_document_viewport_set_offsety(ImageDocument *this, int y);
void image_document_viewport_set_size(ImageDocument *this, int w, int h);
void image_document_viewport_zoom_at(ImageDocument *this, int x, int y, double zoom);
void image_document_viewport_zoom_at_internal(ImageDocument *this, int x, int y, double zoom);
void image_document_viewport_zoom_at_center(ImageDocument *this, double zoom);
_ngg_tuple_update_composite image_document_update_composite(ImageDocument *this, _Bool fillchecker, _Bool fillbg);
struct Rectangle image_document_update_preview(ImageDocument *this);
void image_document_redo(ImageDocument *this);
void image_document_set_bgvisible(ImageDocument *this, _Bool visible);
void image_document_set_fgalpha(ImageDocument *this, double alpha);
void image_fill_checkpattern(Image *img);
int write_image_to_file(Image *img, const char * fname);

#endif /* N96_DRAW_doc_h */
