/* main.c
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include "main.h"

int main(int argc, char * *argv)
{
	gtk_init(0, NULL);
	Test testtype = TEST_NONE;
	if(argc > 2) {
		assert(argv[0]); /* main.ngg:20 */
		fprintf(stderr, "%s: unsupported number of command-line arguments\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	if(argc > 1) {
		char * opt = argv[1];
		if(0 == strcmp(opt, "--static-cursor")) {
		{
			clopts.static_cursor = true;
		}
		}
		else if(0 == strcmp(opt, "--test")) {
		{
			testtype = TEST_UI;
		}
		}
		else if(0 == strcmp(opt, "--test-icons")) {
		{
			testtype = TEST_ICONS;
		}
		}
		else if(0 == strcmp(opt, "--test-sunset")) {
		{
			testtype = TEST_SUNSET;
		}
		}
		else {
		{
			fprintf(stderr, "%s: unknown command-line option: %s\n", argv[0], opt);
			exit(EXIT_FAILURE);
		}
		}
	}


	ImageDocument * doc = malloc(sizeof(ImageDocument));
	if(doc == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_document_construct(doc, DEFWIDTH_DOC, DEFHEIGHT_DOC);

	myapp * app = malloc(sizeof(myapp));
	if(app == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	myapp_construct(app, doc);
	switch(testtype) {
	case TEST_NONE:
	{
		break;
	}
	case TEST_UI:
	{
		testmain_general(app);
		break;
	}
	case TEST_ICONS:
	{
		testmain_draw_icons(app);
		break;
	}
	case TEST_SUNSET:
	{
		testmain_draw_sunset(app);
		break;
	}
	}

	gtk_main();

	return 0;
}

