/* ui.c
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include "ui.h"

#include "warpcore.c"
CommandLineOptions clopts;
double ZOOMSTEP = 0.1;
void myapp_construct(myapp *this, ImageDocument *newdoc)
{
	this->curcursor_is_circle = false;
	this->curcursorsiz = 0;
	this->doc = NULL;
	this->surf = NULL;
	this->disable_brush_preview = false;
	this->was_autoscroll = false;
	this->wnd = (GtkWindow*) (gtk_window_new(GTK_WINDOW_TOPLEVEL));
	gtk_window_set_title(this->wnd, PROGRAM_NAME);
	g_signal_connect((GtkWidget *) this->wnd, "delete-event", G_CALLBACK(on_wnd_deleteevent), this);
	gtk_widget_set_visible((GtkWidget *) this->wnd, false);
	this->accgrp = (GtkAccelGroup*) (gtk_accel_group_new());
	gtk_window_add_accel_group(this->wnd, this->accgrp);
	GtkBox *__ngg_tmp_9_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_VERTICAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_9_bx, 4);

	this->box_main = (GtkBox *) __ngg_tmp_9_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->wnd), GTK_WIDGET(this->box_main));
	gtk_box_set_spacing(this->box_main, 0);
	gtk_container_set_border_width((GtkContainer *) this->box_main, 0);
	this->mbr = (GtkMenuBar*) (gtk_menu_bar_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_main), GTK_WIDGET(this->mbr));
	this->mni_file = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mbr, (GtkWidget*) this->mni_file);
	gtk_menu_item_set_label(this->mni_file, "_File");
	gtk_menu_item_set_use_underline(this->mni_file, true);
	this->mnu_file = (GtkMenu*) (gtk_menu_new());
	gtk_menu_item_set_submenu(this->mni_file, (GtkWidget *) this->mnu_file);
	this->mni_file_new = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_file, (GtkWidget*) this->mni_file_new);
	gtk_menu_item_set_label(this->mni_file_new, "_New");
	gtk_menu_item_set_use_underline(this->mni_file_new, true);
	g_signal_connect((GtkWidget *) this->mni_file_new, "activate", G_CALLBACK(myapp_on_btn_new_doc_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_file_new);
	this->mni_file_export = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_file, (GtkWidget*) this->mni_file_export);
	gtk_menu_item_set_label(this->mni_file_export, "_Export");
	gtk_menu_item_set_use_underline(this->mni_file_export, true);
	g_signal_connect((GtkWidget *) this->mni_file_export, "activate", G_CALLBACK(myapp_on_btn_export_doc_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_file_export);
	this->mni_file_export_layer = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_file, (GtkWidget*) this->mni_file_export_layer);
	gtk_menu_item_set_label(this->mni_file_export_layer, "Export _Layer");
	gtk_menu_item_set_use_underline(this->mni_file_export_layer, true);
	g_signal_connect((GtkWidget *) this->mni_file_export_layer, "activate", G_CALLBACK(myapp_on_btn_export_layer_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_file_export_layer);
	this->mni_file_close_doc = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_file, (GtkWidget*) this->mni_file_close_doc);
	gtk_menu_item_set_label(this->mni_file_close_doc, "_Close Document");
	gtk_menu_item_set_use_underline(this->mni_file_close_doc, true);
	g_signal_connect((GtkWidget *) this->mni_file_close_doc, "activate", G_CALLBACK(myapp_on_btn_close_doc_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_file_close_doc);
	gtk_widget_show((GtkWidget *) this->mnu_file);
	gtk_widget_show((GtkWidget *) this->mni_file);
	this->mni_edit = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mbr, (GtkWidget*) this->mni_edit);
	gtk_menu_item_set_label(this->mni_edit, "_Edit");
	gtk_menu_item_set_use_underline(this->mni_edit, true);
	this->mnu_edit = (GtkMenu*) (gtk_menu_new());
	gtk_menu_item_set_submenu(this->mni_edit, (GtkWidget *) this->mnu_edit);
	this->mni_edit_undo = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_edit, (GtkWidget*) this->mni_edit_undo);
	gtk_menu_item_set_label(this->mni_edit_undo, "_Undo");
	gtk_menu_item_set_use_underline(this->mni_edit_undo, true);
	g_signal_connect((GtkWidget *) this->mni_edit_undo, "activate", G_CALLBACK(myapp_on_btn_undo_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_edit_undo);
	this->mni_edit_redo = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_edit, (GtkWidget*) this->mni_edit_redo);
	gtk_menu_item_set_label(this->mni_edit_redo, "_Redo");
	gtk_menu_item_set_use_underline(this->mni_edit_redo, true);
	g_signal_connect((GtkWidget *) this->mni_edit_redo, "activate", G_CALLBACK(myapp_on_btn_redo_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_edit_redo);
	this->mni_edit_fill_fgcolor = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_edit, (GtkWidget*) this->mni_edit_fill_fgcolor);
	gtk_menu_item_set_label(this->mni_edit_fill_fgcolor, "_Fill with Foreground Colour");
	gtk_menu_item_set_use_underline(this->mni_edit_fill_fgcolor, true);
	g_signal_connect((GtkWidget *) this->mni_edit_fill_fgcolor, "activate", G_CALLBACK(myapp_on_mni_fill_fgcolor_activate), this);
	gtk_widget_show((GtkWidget *) this->mni_edit_fill_fgcolor);
	this->mni_edit_fill_bgcolor = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_edit, (GtkWidget*) this->mni_edit_fill_bgcolor);
	gtk_menu_item_set_label(this->mni_edit_fill_bgcolor, "Fill with Layer _Background");
	gtk_menu_item_set_use_underline(this->mni_edit_fill_bgcolor, true);
	g_signal_connect((GtkWidget *) this->mni_edit_fill_bgcolor, "activate", G_CALLBACK(myapp_on_mni_fill_bgcolor_activate), this);
	gtk_widget_show((GtkWidget *) this->mni_edit_fill_bgcolor);
	this->mni_edit_clear = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_edit, (GtkWidget*) this->mni_edit_clear);
	gtk_menu_item_set_label(this->mni_edit_clear, "_Clear");
	gtk_menu_item_set_use_underline(this->mni_edit_clear, true);
	g_signal_connect((GtkWidget *) this->mni_edit_clear, "activate", G_CALLBACK(myapp_on_btn_clear_layer_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_edit_clear);
	gtk_widget_show((GtkWidget *) this->mnu_edit);
	gtk_widget_show((GtkWidget *) this->mni_edit);
	this->mni_view = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mbr, (GtkWidget*) this->mni_view);
	gtk_menu_item_set_label(this->mni_view, "_View");
	gtk_menu_item_set_use_underline(this->mni_view, true);
	this->mnu_view = (GtkMenu*) (gtk_menu_new());
	gtk_menu_item_set_submenu(this->mni_view, (GtkWidget *) this->mnu_view);
	this->mni_view_zoomin = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_view, (GtkWidget*) this->mni_view_zoomin);
	gtk_menu_item_set_label(this->mni_view_zoomin, "Zoom _In");
	gtk_menu_item_set_use_underline(this->mni_view_zoomin, true);
	g_signal_connect((GtkWidget *) this->mni_view_zoomin, "activate", G_CALLBACK(myapp_on_btn_zoomin_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_view_zoomin);
	this->mni_view_zoomout = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_view, (GtkWidget*) this->mni_view_zoomout);
	gtk_menu_item_set_label(this->mni_view_zoomout, "Zoom _Out");
	gtk_menu_item_set_use_underline(this->mni_view_zoomout, true);
	g_signal_connect((GtkWidget *) this->mni_view_zoomout, "activate", G_CALLBACK(myapp_on_btn_zoomout_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_view_zoomout);
	gtk_widget_show((GtkWidget *) this->mnu_view);
	gtk_widget_show((GtkWidget *) this->mni_view);
	this->mni_tools = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mbr, (GtkWidget*) this->mni_tools);
	gtk_menu_item_set_label(this->mni_tools, "_Tools");
	gtk_menu_item_set_use_underline(this->mni_tools, true);
	this->mnu_tools = (GtkMenu*) (gtk_menu_new());
	gtk_menu_item_set_submenu(this->mni_tools, (GtkWidget *) this->mnu_tools);
	this->mni_tools_pencil = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_pencil);
	gtk_menu_item_set_label(this->mni_tools_pencil, "_Pencil");
	gtk_menu_item_set_use_underline(this->mni_tools_pencil, true);
	g_signal_connect((GtkWidget *) this->mni_tools_pencil, "activate", G_CALLBACK(myapp_on_mni_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_pencil);
	this->mni_tools_brush = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_brush);
	gtk_menu_item_set_label(this->mni_tools_brush, "_Brush");
	gtk_menu_item_set_use_underline(this->mni_tools_brush, true);
	g_signal_connect((GtkWidget *) this->mni_tools_brush, "activate", G_CALLBACK(myapp_on_mni_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_brush);
	this->mni_tools_eraser = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_eraser);
	gtk_menu_item_set_label(this->mni_tools_eraser, "_Eraser");
	gtk_menu_item_set_use_underline(this->mni_tools_eraser, true);
	g_signal_connect((GtkWidget *) this->mni_tools_eraser, "activate", G_CALLBACK(myapp_on_mni_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_eraser);
	this->mni_tools_floodfill = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_floodfill);
	gtk_menu_item_set_label(this->mni_tools_floodfill, "_Fill");
	gtk_menu_item_set_use_underline(this->mni_tools_floodfill, true);
	g_signal_connect((GtkWidget *) this->mni_tools_floodfill, "activate", G_CALLBACK(myapp_on_mni_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_floodfill);
	this->mni_tools_eyedropper = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_eyedropper);
	gtk_menu_item_set_label(this->mni_tools_eyedropper, "_Eyedropper");
	gtk_menu_item_set_use_underline(this->mni_tools_eyedropper, true);
	g_signal_connect((GtkWidget *) this->mni_tools_eyedropper, "activate", G_CALLBACK(myapp_on_mni_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_eyedropper);
	this->mni_tools_bplus = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_bplus);
	gtk_menu_item_set_label(this->mni_tools_bplus, "_Increase Brush Size");
	gtk_menu_item_set_use_underline(this->mni_tools_bplus, true);
	g_signal_connect((GtkWidget *) this->mni_tools_bplus, "activate", G_CALLBACK(myapp_on_btn_bplus_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_bplus);
	this->mni_tools_bminus = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_tools, (GtkWidget*) this->mni_tools_bminus);
	gtk_menu_item_set_label(this->mni_tools_bminus, "_Decrease Brush Size");
	gtk_menu_item_set_use_underline(this->mni_tools_bminus, true);
	g_signal_connect((GtkWidget *) this->mni_tools_bminus, "activate", G_CALLBACK(myapp_on_btn_bminus_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_tools_bminus);
	gtk_widget_show((GtkWidget *) this->mnu_tools);
	gtk_widget_show((GtkWidget *) this->mni_tools);
	this->mni_help = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mbr, (GtkWidget*) this->mni_help);
	gtk_menu_item_set_label(this->mni_help, "_Help");
	gtk_menu_item_set_use_underline(this->mni_help, true);
	this->mnu_help = (GtkMenu*) (gtk_menu_new());
	gtk_menu_item_set_submenu(this->mni_help, (GtkWidget *) this->mnu_help);
	this->mni_help_about = (GtkMenuItem*) (gtk_menu_item_new());
	gtk_menu_shell_append((GtkMenuShell*) this->mnu_help, (GtkWidget*) this->mni_help_about);
	gtk_menu_item_set_label(this->mni_help_about, "_About");
	gtk_menu_item_set_use_underline(this->mni_help_about, true);
	g_signal_connect((GtkWidget *) this->mni_help_about, "activate", G_CALLBACK(myapp_on_btn_help_about_clicked), this);
	gtk_widget_show((GtkWidget *) this->mni_help_about);
	gtk_widget_show((GtkWidget *) this->mnu_help);
	gtk_widget_show((GtkWidget *) this->mni_help);
	gtk_widget_show((GtkWidget *) this->mbr);
	this->sw_controls2 = (GtkScrolledWindow*) (gtk_scrolled_window_new(NULL, NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_main), GTK_WIDGET(this->sw_controls2));
	gtk_scrolled_window_set_propagate_natural_height(this->sw_controls2, true);
	GtkBox *__ngg_tmp_10_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_10_bx, 4);

	this->box_controls2 = (GtkBox *) __ngg_tmp_10_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->sw_controls2), GTK_WIDGET(this->box_controls2));
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls2, false);
	this->fme_tools = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_tools));
	gtk_frame_set_label((GtkFrame *) this->fme_tools, "Tools");
	GtkBox *__ngg_tmp_11_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_11_bx, 4);

	this->box_tools = (GtkBox *) __ngg_tmp_11_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_tools), GTK_WIDGET(this->box_tools));
	this->btn_pencil = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_tools), GTK_WIDGET(this->btn_pencil));
	gtk_button_set_label((GtkButton *) this->btn_pencil, "Pencil");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_pencil, false);
	g_signal_connect((GtkWidget *) this->btn_pencil, "toggled", G_CALLBACK(myapp_on_btn_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_pencil);
	this->btn_brush = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_tools), GTK_WIDGET(this->btn_brush));
	gtk_button_set_label((GtkButton *) this->btn_brush, "Brush");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_brush, false);
	g_signal_connect((GtkWidget *) this->btn_brush, "toggled", G_CALLBACK(myapp_on_btn_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_brush);
	this->btn_eraser = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_tools), GTK_WIDGET(this->btn_eraser));
	gtk_button_set_label((GtkButton *) this->btn_eraser, "Eraser");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_eraser, false);
	g_signal_connect((GtkWidget *) this->btn_eraser, "toggled", G_CALLBACK(myapp_on_btn_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_eraser);
	this->btn_eyedropper = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_tools), GTK_WIDGET(this->btn_eyedropper));
	gtk_button_set_label((GtkButton *) this->btn_eyedropper, "Picker");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_eyedropper, false);
	g_signal_connect((GtkWidget *) this->btn_eyedropper, "toggled", G_CALLBACK(myapp_on_btn_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_eyedropper);
	this->btn_floodfill = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_tools), GTK_WIDGET(this->btn_floodfill));
	gtk_button_set_label((GtkButton *) this->btn_floodfill, "Fill");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_floodfill, false);
	g_signal_connect((GtkWidget *) this->btn_floodfill, "toggled", G_CALLBACK(myapp_on_btn_tool_x_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_floodfill);
	gtk_widget_show((GtkWidget *) this->box_tools);
	gtk_widget_show((GtkWidget *) this->fme_tools);
	this->fme_strokeopts = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_strokeopts));
	gtk_frame_set_label((GtkFrame *) this->fme_strokeopts, "Colour");
	GtkBox *__ngg_tmp_12_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_12_bx, 4);

	this->box_strokeopts = (GtkBox *) __ngg_tmp_12_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_strokeopts), GTK_WIDGET(this->box_strokeopts));
	this->btn_color = (GtkColorButton*) (gtk_color_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_strokeopts), GTK_WIDGET(this->btn_color));
	g_signal_connect((GtkWidget *) this->btn_color, "color-set", G_CALLBACK(myapp_on_btn_color_set), this);
	gtk_widget_show((GtkWidget *) this->btn_color);
	gtk_widget_show((GtkWidget *) this->box_strokeopts);
	gtk_widget_show((GtkWidget *) this->fme_strokeopts);
	this->fme_brushsiz = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_brushsiz));
	gtk_frame_set_label((GtkFrame *) this->fme_brushsiz, "Brush Size");
	GtkBox *__ngg_tmp_13_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_13_bx, 4);

	this->box_brushsiz = (GtkBox *) __ngg_tmp_13_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_brushsiz), GTK_WIDGET(this->box_brushsiz));
	this->btn_bplus = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushsiz), GTK_WIDGET(this->btn_bplus));
	gtk_button_set_label(this->btn_bplus, "B+");
	g_signal_connect((GtkWidget *) this->btn_bplus, "clicked", G_CALLBACK(myapp_on_btn_bplus_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_bplus);
	this->btn_bminus = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushsiz), GTK_WIDGET(this->btn_bminus));
	gtk_button_set_label(this->btn_bminus, "B-");
	g_signal_connect((GtkWidget *) this->btn_bminus, "clicked", G_CALLBACK(myapp_on_btn_bminus_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_bminus);
	this->lbl_brush_size = (GtkLabel*) (gtk_label_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushsiz), GTK_WIDGET(this->lbl_brush_size));
	gtk_widget_set_size_request((GtkWidget*) this->lbl_brush_size, MINWIDTH_LBL_BRUSH_SIZE, -1);
	gtk_widget_show((GtkWidget *) this->lbl_brush_size);
	gtk_widget_show((GtkWidget *) this->box_brushsiz);
	gtk_widget_show((GtkWidget *) this->fme_brushsiz);
	this->fme_brushpreview = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_brushpreview));
	gtk_frame_set_label((GtkFrame *) this->fme_brushpreview, "Brush Preview");

	ImageView * _ngg_tmp_14 = malloc(sizeof(ImageView));
	if(_ngg_tmp_14 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_view_construct(_ngg_tmp_14);
	this->iv_brushpreview = _ngg_tmp_14;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_brushpreview), GTK_WIDGET(image_view_get_native_widget(this->iv_brushpreview)));

	Image * _ngg_tmp_15 = malloc(sizeof(Image));
	if(_ngg_tmp_15 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_construct(_ngg_tmp_15, 120, 40, 255);
	image_view_set_image(this->iv_brushpreview, _ngg_tmp_15);
	image_view_show(this->iv_brushpreview);
	gtk_widget_show((GtkWidget *) this->fme_brushpreview);
	this->fme_brushes = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_brushes));
	gtk_frame_set_label((GtkFrame *) this->fme_brushes, "Brush Presets");
	GtkBox *__ngg_tmp_16_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_16_bx, 4);

	this->box_brushes = (GtkBox *) __ngg_tmp_16_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_brushes), GTK_WIDGET(this->box_brushes));
	this->btn_prebrush_ballpoint = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushes), GTK_WIDGET(this->btn_prebrush_ballpoint));
	gtk_button_set_label(this->btn_prebrush_ballpoint, "Ballpoint");
	gtk_button_set_image(this->btn_prebrush_ballpoint, brushpreview_widget_new(prebrush_ballpoint()));
	gtk_button_set_image_position(this->btn_prebrush_ballpoint, GTK_POS_LEFT);
	g_signal_connect((GtkWidget *) this->btn_prebrush_ballpoint, "clicked", G_CALLBACK(myapp_on_btn_prebrush_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_prebrush_ballpoint);
	this->btn_prebrush_charcoal = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushes), GTK_WIDGET(this->btn_prebrush_charcoal));
	gtk_button_set_label(this->btn_prebrush_charcoal, "Charcoal");
	gtk_button_set_image(this->btn_prebrush_charcoal, brushpreview_widget_new(prebrush_charcoal()));
	gtk_button_set_image_position(this->btn_prebrush_charcoal, GTK_POS_LEFT);
	g_signal_connect((GtkWidget *) this->btn_prebrush_charcoal, "clicked", G_CALLBACK(myapp_on_btn_prebrush_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_prebrush_charcoal);
	this->btn_prebrush_round = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushes), GTK_WIDGET(this->btn_prebrush_round));
	gtk_button_set_label(this->btn_prebrush_round, "Round");
	gtk_button_set_image(this->btn_prebrush_round, brushpreview_widget_new(prebrush_round()));
	gtk_button_set_image_position(this->btn_prebrush_round, GTK_POS_LEFT);
	g_signal_connect((GtkWidget *) this->btn_prebrush_round, "clicked", G_CALLBACK(myapp_on_btn_prebrush_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_prebrush_round);
	this->btn_prebrush_round_pressure_size = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushes), GTK_WIDGET(this->btn_prebrush_round_pressure_size));
	gtk_button_set_label(this->btn_prebrush_round_pressure_size, "Round Pr. Size");
	gtk_button_set_image(this->btn_prebrush_round_pressure_size, brushpreview_widget_new(prebrush_round_pressure_size()));
	gtk_button_set_image_position(this->btn_prebrush_round_pressure_size, GTK_POS_LEFT);
	g_signal_connect((GtkWidget *) this->btn_prebrush_round_pressure_size, "clicked", G_CALLBACK(myapp_on_btn_prebrush_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_prebrush_round_pressure_size);
	this->btn_prebrush_soft_round = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushes), GTK_WIDGET(this->btn_prebrush_soft_round));
	gtk_button_set_label(this->btn_prebrush_soft_round, "Soft Round");
	gtk_button_set_image(this->btn_prebrush_soft_round, brushpreview_widget_new(prebrush_soft_round()));
	gtk_button_set_image_position(this->btn_prebrush_soft_round, GTK_POS_LEFT);
	g_signal_connect((GtkWidget *) this->btn_prebrush_soft_round, "clicked", G_CALLBACK(myapp_on_btn_prebrush_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_prebrush_soft_round);
	gtk_widget_show((GtkWidget *) this->box_brushes);
	gtk_widget_show((GtkWidget *) this->fme_brushes);
	this->fme_history = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->fme_history));
	gtk_frame_set_label((GtkFrame *) this->fme_history, "History");
	GtkBox *__ngg_tmp_17_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_17_bx, 4);

	this->box_history = (GtkBox *) __ngg_tmp_17_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_history), GTK_WIDGET(this->box_history));
	this->btn_undo = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_undo));
	gtk_button_set_label(this->btn_undo, "Undo");
	gtk_widget_set_sensitive((GtkWidget *) this->btn_undo, false);
	g_signal_connect((GtkWidget *) this->btn_undo, "clicked", G_CALLBACK(myapp_on_btn_undo_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_undo);
	this->btn_redo = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_redo));
	gtk_button_set_label(this->btn_redo, "Redo");
	gtk_widget_set_sensitive((GtkWidget *) this->btn_redo, false);
	g_signal_connect((GtkWidget *) this->btn_redo, "clicked", G_CALLBACK(myapp_on_btn_redo_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_redo);
	gtk_widget_show((GtkWidget *) this->box_history);
	gtk_widget_show((GtkWidget *) this->fme_history);
	this->chk_advanced = (GtkCheckButton*) (gtk_check_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls2), GTK_WIDGET(this->chk_advanced));
	gtk_button_set_label((GtkButton *) this->chk_advanced, "Advanced");
	g_signal_connect((GtkWidget *) this->chk_advanced, "toggled", G_CALLBACK(myapp_on_chk_advanced_toggled), this);
	gtk_widget_show((GtkWidget *) this->chk_advanced);
	gtk_widget_show((GtkWidget *) this->box_controls2);
	gtk_widget_show((GtkWidget *) this->sw_controls2);
	this->sw_controls3 = (GtkScrolledWindow*) (gtk_scrolled_window_new(NULL, NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_main), GTK_WIDGET(this->sw_controls3));
	gtk_scrolled_window_set_propagate_natural_height(this->sw_controls3, true);
	gtk_widget_set_visible((GtkWidget *) this->sw_controls3, false);
	GtkBox *__ngg_tmp_18_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_18_bx, 4);

	this->box_controls3 = (GtkBox *) __ngg_tmp_18_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->sw_controls3), GTK_WIDGET(this->box_controls3));
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls3, false);
	this->fme_brushopts = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls3), GTK_WIDGET(this->fme_brushopts));
	gtk_frame_set_label((GtkFrame *) this->fme_brushopts, "Brush Options");
	GtkBox *__ngg_tmp_19_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_19_bx, 4);

	this->box_brushopts = (GtkBox *) __ngg_tmp_19_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_brushopts), GTK_WIDGET(this->box_brushopts));
	this->btn_brushtype_circle1 = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(this->btn_brushtype_circle1));
	gtk_button_set_label((GtkButton *) this->btn_brushtype_circle1, "Circle 1");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_brushtype_circle1, false);
	g_signal_connect((GtkWidget *) this->btn_brushtype_circle1, "toggled", G_CALLBACK(myapp_on_btn_brushtype_circlex_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_brushtype_circle1);
	this->btn_brushtype_circle2 = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(this->btn_brushtype_circle2));
	gtk_button_set_label((GtkButton *) this->btn_brushtype_circle2, "Circle 2");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_brushtype_circle2, false);
	g_signal_connect((GtkWidget *) this->btn_brushtype_circle2, "toggled", G_CALLBACK(myapp_on_btn_brushtype_circlex_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_brushtype_circle2);
	this->btn_brushtype_circle1noise = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(this->btn_brushtype_circle1noise));
	gtk_button_set_label((GtkButton *) this->btn_brushtype_circle1noise, "Circle 1 Noise");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_brushtype_circle1noise, false);
	g_signal_connect((GtkWidget *) this->btn_brushtype_circle1noise, "toggled", G_CALLBACK(myapp_on_btn_brushtype_circlex_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_brushtype_circle1noise);
	this->btn_brushtype_randsquare = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(this->btn_brushtype_randsquare));
	gtk_button_set_label((GtkButton *) this->btn_brushtype_randsquare, "Rand. Square");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_brushtype_randsquare, false);
	g_signal_connect((GtkWidget *) this->btn_brushtype_randsquare, "toggled", G_CALLBACK(myapp_on_btn_brushtype_randsquare_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_brushtype_randsquare);
	this->chk_pressure = (GtkCheckButton*) (gtk_check_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(this->chk_pressure));
	gtk_button_set_label((GtkButton *) this->chk_pressure, "Pressure");
	g_signal_connect((GtkWidget *) this->chk_pressure, "toggled", G_CALLBACK(myapp_on_chk_pressure_toggled), this);
	gtk_widget_show((GtkWidget *) this->chk_pressure);

	EntrySlider * _ngg_tmp_20 = malloc(sizeof(EntrySlider));
	if(_ngg_tmp_20 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	entry_slider_construct(_ngg_tmp_20);
	this->sldr_brush_opacity = _ngg_tmp_20;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(entry_slider_get_native_widget(this->sldr_brush_opacity)));
	entry_slider_set_label(this->sldr_brush_opacity, "Opacity: ");
	entry_slider_set_onchanged(this->sldr_brush_opacity, myapp_on_sldr_brush_opacity_changed, this);
	entry_slider_show(this->sldr_brush_opacity);

	EntrySlider * _ngg_tmp_21 = malloc(sizeof(EntrySlider));
	if(_ngg_tmp_21 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	entry_slider_construct(_ngg_tmp_21);
	this->sldr_brush_softness = _ngg_tmp_21;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_brushopts), GTK_WIDGET(entry_slider_get_native_widget(this->sldr_brush_softness)));
	entry_slider_set_label(this->sldr_brush_softness, "Softness: ");
	entry_slider_set_onchanged(this->sldr_brush_softness, myapp_on_sldr_brush_softness_changed, this);
	entry_slider_show(this->sldr_brush_softness);
	gtk_widget_show((GtkWidget *) this->box_brushopts);
	gtk_widget_show((GtkWidget *) this->fme_brushopts);
	this->fme_strokeopts = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls3), GTK_WIDGET(this->fme_strokeopts));
	gtk_frame_set_label((GtkFrame *) this->fme_strokeopts, "Stroke Options");
	GtkBox *__ngg_tmp_22_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_22_bx, 4);

	this->box_strokeopts = (GtkBox *) __ngg_tmp_22_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_strokeopts), GTK_WIDGET(this->box_strokeopts));

	EntrySlider * _ngg_tmp_23 = malloc(sizeof(EntrySlider));
	if(_ngg_tmp_23 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	entry_slider_construct(_ngg_tmp_23);
	this->sldr_stroke_smoothness = _ngg_tmp_23;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_strokeopts), GTK_WIDGET(entry_slider_get_native_widget(this->sldr_stroke_smoothness)));
	entry_slider_set_label(this->sldr_stroke_smoothness, "Smoothness: ");
	entry_slider_set_onchanged(this->sldr_stroke_smoothness, myapp_on_sldr_stroke_smoothness_changed, this);
	entry_slider_show(this->sldr_stroke_smoothness);
	this->chk_subpixel_when_zoomed = (GtkCheckButton*) (gtk_check_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_strokeopts), GTK_WIDGET(this->chk_subpixel_when_zoomed));
	gtk_button_set_label((GtkButton *) this->chk_subpixel_when_zoomed, "Subpixel");
	g_signal_connect((GtkWidget *) this->chk_subpixel_when_zoomed, "toggled", G_CALLBACK(myapp_on_chk_subpixel_when_zoomed_toggled), this);
	gtk_widget_show((GtkWidget *) this->chk_subpixel_when_zoomed);
	gtk_widget_show((GtkWidget *) this->box_strokeopts);
	gtk_widget_show((GtkWidget *) this->fme_strokeopts);
	this->fme_about = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls3), GTK_WIDGET(this->fme_about));
	GtkBox *__ngg_tmp_24_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_24_bx, 4);

	this->hbx_about = (GtkBox *) __ngg_tmp_24_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_about), GTK_WIDGET(this->hbx_about));
	this->btn_help_about = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->hbx_about), GTK_WIDGET(this->btn_help_about));
	gtk_button_set_label(this->btn_help_about, "_About");
	gtk_button_set_use_underline(this->btn_help_about, true);
	g_signal_connect((GtkWidget *) this->btn_help_about, "clicked", G_CALLBACK(myapp_on_btn_help_about_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_help_about);
	gtk_widget_show((GtkWidget *) this->hbx_about);
	gtk_widget_show((GtkWidget *) this->fme_about);
	gtk_widget_show((GtkWidget *) this->box_controls3);
	this->grd_viewport = (GtkGrid*) (gtk_grid_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_main), GTK_WIDGET(this->grd_viewport));
	this->da = (GtkDrawingArea*) (gtk_drawing_area_new());
	gtk_grid_attach(this->grd_viewport, this->da, 0, 0, 1, 1);
	gtk_widget_set_sensitive((GtkWidget *) this->da, false);
	g_signal_connect((GtkWidget *) this->da, "size-allocate", G_CALLBACK(myapp_on_da_sizeallocate), this);
	gtk_widget_set_hexpand((GtkWidget *) this->da, true);
	gtk_widget_set_vexpand((GtkWidget *) this->da, true);
	gtk_widget_show((GtkWidget *) this->da);
	this->vscroll = (GtkScrollbar *) (GtkScrollbar*) (gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, NULL));
	gtk_grid_attach(this->grd_viewport, this->vscroll, 1, 0, 1, 1);
	gtk_range_set_slider_size_fixed((GtkRange *) this->vscroll, false);
	gtk_adjustment_set_step_increment(gtk_range_get_adjustment((GtkRange *) this->vscroll), 1);
	g_signal_connect((GtkWidget *) this->vscroll, "value-changed", G_CALLBACK(myapp_on_vscroll_valuechanged), this);
	gtk_widget_show((GtkWidget *) this->vscroll);
	this->hscroll = (GtkScrollbar*) (gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL, NULL));
	gtk_grid_attach(this->grd_viewport, this->hscroll, 0, 1, 1, 1);
	gtk_widget_set_hexpand((GtkWidget *) this->hscroll, true);
	gtk_range_set_slider_size_fixed((GtkRange *) this->hscroll, false);
	gtk_adjustment_set_step_increment(gtk_range_get_adjustment((GtkRange *) this->hscroll), 1);
	g_signal_connect((GtkWidget *) this->hscroll, "value-changed", G_CALLBACK(myapp_on_hscroll_valuechanged), this);
	gtk_widget_show((GtkWidget *) this->hscroll);
	gtk_widget_show((GtkWidget *) this->grd_viewport);
	this->grd_bottom = (GtkGrid*) (gtk_grid_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_main), GTK_WIDGET(this->grd_bottom));
	GtkBox *__ngg_tmp_25_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_25_bx, 4);

	this->box_newdoc = (GtkBox *) __ngg_tmp_25_bx;
	gtk_grid_attach(this->grd_bottom, this->box_newdoc, 0, 0, 1, 1);
	this->fme_newdoc = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_newdoc), GTK_WIDGET(this->fme_newdoc));
	GtkBox *__ngg_tmp_26_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_26_bx, 4);

	this->hbx_newdoc = (GtkBox *) __ngg_tmp_26_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_newdoc), GTK_WIDGET(this->hbx_newdoc));
	this->btn_export_doc = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->hbx_newdoc), GTK_WIDGET(this->btn_export_doc));
	gtk_button_set_label(this->btn_export_doc, "Export");
	g_signal_connect((GtkWidget *) this->btn_export_doc, "clicked", G_CALLBACK(myapp_on_btn_export_doc_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_export_doc);
	gtk_widget_show((GtkWidget *) this->hbx_newdoc);
	gtk_widget_show((GtkWidget *) this->fme_newdoc);
	gtk_widget_show((GtkWidget *) this->box_newdoc);
	this->sw_controls1 = (GtkScrolledWindow*) (gtk_scrolled_window_new(NULL, NULL));
	gtk_grid_attach(this->grd_bottom, this->sw_controls1, 1, 0, 1, 1);
	gtk_scrolled_window_set_propagate_natural_height(this->sw_controls1, true);
	gtk_widget_set_hexpand((GtkWidget *) this->sw_controls1, true);
	GtkBox *__ngg_tmp_27_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_27_bx, 4);

	this->box_controls1 = (GtkBox *) __ngg_tmp_27_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->sw_controls1), GTK_WIDGET(this->box_controls1));
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls1, false);
	this->fme_layers = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(this->fme_layers));
	gtk_frame_set_label((GtkFrame *) this->fme_layers, "Layers");
	GtkBox *__ngg_tmp_28_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_28_bx, 4);

	this->box_layers = (GtkBox *) __ngg_tmp_28_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_layers), GTK_WIDGET(this->box_layers));
	this->btn_fglayer = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_layers), GTK_WIDGET(this->btn_fglayer));
	gtk_button_set_label((GtkButton *) this->btn_fglayer, "FG");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_fglayer, false);
	g_signal_connect((GtkWidget *) this->btn_fglayer, "toggled", G_CALLBACK(myapp_on_btn_fglayer_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_fglayer);
	this->btn_bglayer = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_layers), GTK_WIDGET(this->btn_bglayer));
	gtk_button_set_label((GtkButton *) this->btn_bglayer, "BG");
	gtk_toggle_button_set_mode((GtkToggleButton *) this->btn_bglayer, false);
	g_signal_connect((GtkWidget *) this->btn_bglayer, "toggled", G_CALLBACK(myapp_on_btn_bglayer_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_bglayer);
	gtk_widget_show((GtkWidget *) this->box_layers);
	gtk_widget_show((GtkWidget *) this->fme_layers);
	this->fme_layer = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(this->fme_layer));
	gtk_frame_set_label((GtkFrame *) this->fme_layer, "Layer Actions");
	GtkBox *__ngg_tmp_29_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_29_bx, 4);

	this->box_layer = (GtkBox *) __ngg_tmp_29_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_layer), GTK_WIDGET(this->box_layer));
	this->btn_layer_fillbgcolor = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_layer), GTK_WIDGET(this->btn_layer_fillbgcolor));
	gtk_button_set_label(this->btn_layer_fillbgcolor, "Fill BG Colour");
	g_signal_connect((GtkWidget *) this->btn_layer_fillbgcolor, "clicked", G_CALLBACK(myapp_on_mni_fill_bgcolor_activate), this);
	gtk_widget_show((GtkWidget *) this->btn_layer_fillbgcolor);
	this->btn_clear_layer = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_layer), GTK_WIDGET(this->btn_clear_layer));
	gtk_button_set_label(this->btn_clear_layer, "Clear");
	g_signal_connect((GtkWidget *) this->btn_clear_layer, "clicked", G_CALLBACK(myapp_on_btn_clear_layer_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_clear_layer);
	gtk_widget_show((GtkWidget *) this->box_layer);
	gtk_widget_show((GtkWidget *) this->fme_layer);
	this->fme_fg_visibility = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(this->fme_fg_visibility));
	gtk_frame_set_label((GtkFrame *) this->fme_fg_visibility, "FG Visibility");
	GtkBox *__ngg_tmp_30_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_30_bx, 4);

	this->box_fg_visibility = (GtkBox *) __ngg_tmp_30_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_fg_visibility), GTK_WIDGET(this->box_fg_visibility));
	this->btn_fgopacity100 = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_fg_visibility), GTK_WIDGET(this->btn_fgopacity100));
	gtk_button_set_label((GtkButton *) this->btn_fgopacity100, "100%");
	g_signal_connect((GtkWidget *) this->btn_fgopacity100, "toggled", G_CALLBACK(myapp_on_btn_fgopacity100_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_fgopacity100);
	this->btn_fgopacity50 = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_fg_visibility), GTK_WIDGET(this->btn_fgopacity50));
	gtk_button_set_label((GtkButton *) this->btn_fgopacity50, "50%");
	g_signal_connect((GtkWidget *) this->btn_fgopacity50, "toggled", G_CALLBACK(myapp_on_btn_fgopacity50_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_fgopacity50);
	this->btn_fgopacity0 = (GtkRadioButton*) (gtk_radio_button_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_fg_visibility), GTK_WIDGET(this->btn_fgopacity0));
	gtk_button_set_label((GtkButton *) this->btn_fgopacity0, "0%");
	g_signal_connect((GtkWidget *) this->btn_fgopacity0, "toggled", G_CALLBACK(myapp_on_btn_fgopacity0_toggled), this);
	gtk_widget_show((GtkWidget *) this->btn_fgopacity0);
	gtk_widget_show((GtkWidget *) this->box_fg_visibility);
	gtk_widget_show((GtkWidget *) this->fme_fg_visibility);
	this->fme_bg_visibility = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(this->fme_bg_visibility));
	gtk_frame_set_label((GtkFrame *) this->fme_bg_visibility, "BG");
	GtkBox *__ngg_tmp_31_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_31_bx, 4);

	this->box_bg_visibility = (GtkBox *) __ngg_tmp_31_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_bg_visibility), GTK_WIDGET(this->box_bg_visibility));
	this->chk_bg_visible = (GtkCheckButton*) (gtk_check_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_bg_visibility), GTK_WIDGET(this->chk_bg_visible));
	gtk_button_set_label((GtkButton *) this->chk_bg_visible, "Visible");
	g_signal_connect((GtkWidget *) this->chk_bg_visible, "toggled", G_CALLBACK(myapp_on_chk_bg_visible_toggled), this);
	gtk_widget_show((GtkWidget *) this->chk_bg_visible);
	gtk_widget_show((GtkWidget *) this->box_bg_visibility);
	gtk_widget_show((GtkWidget *) this->fme_bg_visibility);

	QuickPalette * _ngg_tmp_32 = malloc(sizeof(QuickPalette));
	if(_ngg_tmp_32 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	quick_palette_construct(_ngg_tmp_32);
	this->quickpalette = _ngg_tmp_32;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(image_view_get_native_widget((ImageView *) this->quickpalette)));
	quick_palette_set_hexpand(this->quickpalette, true);
	this->quickpalette->oncolorset_callback = myapp_on_quickpalette_colorset;
	this->quickpalette->oncolorset_data = this;
	image_view_show((ImageView *) this->quickpalette);
	this->fme_zoom = (GtkFrame*) (gtk_frame_new(""));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_controls1), GTK_WIDGET(this->fme_zoom));
	gtk_frame_set_label((GtkFrame *) this->fme_zoom, "Zoom");
	GtkBox *__ngg_tmp_33_bx = (GtkBox*) (gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
	gtk_container_set_border_width((GtkContainer *) __ngg_tmp_33_bx, 4);

	this->box_history = (GtkBox *) __ngg_tmp_33_bx;
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->fme_zoom), GTK_WIDGET(this->box_history));
	this->btn_center = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_center));
	gtk_button_set_label(this->btn_center, "C");
	g_signal_connect((GtkWidget *) this->btn_center, "clicked", G_CALLBACK(myapp_on_btn_center_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_center);
	this->btn_zoom100 = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_zoom100));
	gtk_button_set_label(this->btn_zoom100, "100%");
	g_signal_connect((GtkWidget *) this->btn_zoom100, "clicked", G_CALLBACK(myapp_on_btn_zoom100_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_zoom100);
	this->btn_zoomin = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_zoomin));
	gtk_button_set_label(this->btn_zoomin, "Z+");
	gtk_widget_set_sensitive((GtkWidget *) this->btn_zoomin, false);
	g_signal_connect((GtkWidget *) this->btn_zoomin, "clicked", G_CALLBACK(myapp_on_btn_zoomin_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_zoomin);
	this->btn_zoomout = (GtkButton*) (gtk_button_new());
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->btn_zoomout));
	gtk_button_set_label(this->btn_zoomout, "Z-");
	gtk_widget_set_sensitive((GtkWidget *) this->btn_zoomout, false);
	g_signal_connect((GtkWidget *) this->btn_zoomout, "clicked", G_CALLBACK(myapp_on_btn_zoomout_clicked), this);
	gtk_widget_show((GtkWidget *) this->btn_zoomout);
	this->lbl_zoom = (GtkLabel*) (gtk_label_new(NULL));
	gtk_container_add(GTK_CONTAINER((GtkWidget *) this->box_history), GTK_WIDGET(this->lbl_zoom));
	gtk_widget_set_size_request((GtkWidget*) this->lbl_zoom, MINWIDTH_LBL_ZOOM_SIZE, -1);
	gtk_widget_show((GtkWidget *) this->lbl_zoom);
	gtk_widget_show((GtkWidget *) this->box_history);
	gtk_widget_show((GtkWidget *) this->fme_zoom);
	gtk_widget_show((GtkWidget *) this->box_controls1);
	gtk_widget_show((GtkWidget *) this->sw_controls1);
	gtk_widget_show((GtkWidget *) this->grd_bottom);
	gtk_widget_show((GtkWidget *) this->box_main);
	unsigned int __ngg_tmp_34_accelkey = 0;
	int __ngg_tmp_34_accelmods = 0;
	gtk_accelerator_parse("<Ctrl>n", &__ngg_tmp_34_accelkey, &__ngg_tmp_34_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_file_new, "activate", this->accgrp, __ngg_tmp_34_accelkey, __ngg_tmp_34_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_35_accelkey = 0;
	int __ngg_tmp_35_accelmods = 0;
	gtk_accelerator_parse("<Ctrl>e", &__ngg_tmp_35_accelkey, &__ngg_tmp_35_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_file_export, "activate", this->accgrp, __ngg_tmp_35_accelkey, __ngg_tmp_35_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_36_accelkey = 0;
	int __ngg_tmp_36_accelmods = 0;
	gtk_accelerator_parse("<Ctrl>z", &__ngg_tmp_36_accelkey, &__ngg_tmp_36_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_edit_undo, "activate", this->accgrp, __ngg_tmp_36_accelkey, __ngg_tmp_36_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_37_accelkey = 0;
	int __ngg_tmp_37_accelmods = 0;
	gtk_accelerator_parse("<Ctrl><Shift>z", &__ngg_tmp_37_accelkey, &__ngg_tmp_37_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_edit_redo, "activate", this->accgrp, __ngg_tmp_37_accelkey, __ngg_tmp_37_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_38_accelkey = 0;
	int __ngg_tmp_38_accelmods = 0;
	gtk_accelerator_parse("Delete", &__ngg_tmp_38_accelkey, &__ngg_tmp_38_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_edit_clear, "activate", this->accgrp, __ngg_tmp_38_accelkey, __ngg_tmp_38_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_39_accelkey = 0;
	int __ngg_tmp_39_accelmods = 0;
	gtk_accelerator_parse("BackSpace", &__ngg_tmp_39_accelkey, &__ngg_tmp_39_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_edit_fill_bgcolor, "activate", this->accgrp, __ngg_tmp_39_accelkey, __ngg_tmp_39_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_40_accelkey = 0;
	int __ngg_tmp_40_accelmods = 0;
	gtk_accelerator_parse("<Shift>BackSpace", &__ngg_tmp_40_accelkey, &__ngg_tmp_40_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_edit_fill_fgcolor, "activate", this->accgrp, __ngg_tmp_40_accelkey, __ngg_tmp_40_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_41_accelkey = 0;
	int __ngg_tmp_41_accelmods = 0;
	gtk_accelerator_parse("<Ctrl>equal", &__ngg_tmp_41_accelkey, &__ngg_tmp_41_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_view_zoomin, "activate", this->accgrp, __ngg_tmp_41_accelkey, __ngg_tmp_41_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_42_accelkey = 0;
	int __ngg_tmp_42_accelmods = 0;
	gtk_accelerator_parse("<Ctrl>minus", &__ngg_tmp_42_accelkey, &__ngg_tmp_42_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_view_zoomout, "activate", this->accgrp, __ngg_tmp_42_accelkey, __ngg_tmp_42_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_43_accelkey = 0;
	int __ngg_tmp_43_accelmods = 0;
	gtk_accelerator_parse("b", &__ngg_tmp_43_accelkey, &__ngg_tmp_43_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_brush, "activate", this->accgrp, __ngg_tmp_43_accelkey, __ngg_tmp_43_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_44_accelkey = 0;
	int __ngg_tmp_44_accelmods = 0;
	gtk_accelerator_parse("e", &__ngg_tmp_44_accelkey, &__ngg_tmp_44_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_eraser, "activate", this->accgrp, __ngg_tmp_44_accelkey, __ngg_tmp_44_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_45_accelkey = 0;
	int __ngg_tmp_45_accelmods = 0;
	gtk_accelerator_parse("p", &__ngg_tmp_45_accelkey, &__ngg_tmp_45_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_eyedropper, "activate", this->accgrp, __ngg_tmp_45_accelkey, __ngg_tmp_45_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_46_accelkey = 0;
	int __ngg_tmp_46_accelmods = 0;
	gtk_accelerator_parse("f", &__ngg_tmp_46_accelkey, &__ngg_tmp_46_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_floodfill, "activate", this->accgrp, __ngg_tmp_46_accelkey, __ngg_tmp_46_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_47_accelkey = 0;
	int __ngg_tmp_47_accelmods = 0;
	gtk_accelerator_parse("n", &__ngg_tmp_47_accelkey, &__ngg_tmp_47_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_pencil, "activate", this->accgrp, __ngg_tmp_47_accelkey, __ngg_tmp_47_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_48_accelkey = 0;
	int __ngg_tmp_48_accelmods = 0;
	gtk_accelerator_parse("bracketleft", &__ngg_tmp_48_accelkey, &__ngg_tmp_48_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_bminus, "activate", this->accgrp, __ngg_tmp_48_accelkey, __ngg_tmp_48_accelmods, GTK_ACCEL_VISIBLE);
	unsigned int __ngg_tmp_49_accelkey = 0;
	int __ngg_tmp_49_accelmods = 0;
	gtk_accelerator_parse("bracketright", &__ngg_tmp_49_accelkey, &__ngg_tmp_49_accelmods);
	gtk_widget_add_accelerator((GtkWidget *) this->mni_tools_bplus, "activate", this->accgrp, __ngg_tmp_49_accelkey, __ngg_tmp_49_accelmods, GTK_ACCEL_VISIBLE);
	gtk_radio_button_join_group(this->btn_bglayer, this->btn_fglayer);
	gtk_radio_button_join_group(this->btn_eraser, this->btn_pencil);
	gtk_radio_button_join_group(this->btn_brush, this->btn_pencil);
	gtk_radio_button_join_group(this->btn_eyedropper, this->btn_pencil);
	gtk_radio_button_join_group(this->btn_floodfill, this->btn_pencil);
	gtk_radio_button_join_group(this->btn_brushtype_circle1, this->btn_brushtype_randsquare);
	gtk_radio_button_join_group(this->btn_brushtype_circle1noise, this->btn_brushtype_randsquare);
	gtk_radio_button_join_group(this->btn_brushtype_circle2, this->btn_brushtype_randsquare);
	gtk_radio_button_join_group(this->btn_fgopacity50, this->btn_fgopacity100);
	gtk_radio_button_join_group(this->btn_fgopacity0, this->btn_fgopacity100);
	myapp_reserve_cairo_surface(this, DEFWIDTH_CAIRO, DEFHEIGHT_CAIRO);
	gtk_widget_set_size_request((GtkWidget*) this->wnd, imax2(DEFWIDTH_DOC, MINWIDTH_MWND), imax2(DEFHEIGHT_DOC, MINHEIGHT_MWND));
	gtk_window_set_default_size(this->wnd, DEFWIDTH_MWND, DEFHEIGHT_MWND);
	gtk_widget_show((GtkWidget *) this->wnd);
	gtk_widget_add_events((GtkWidget *) this->da, (((GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK) | GDK_POINTER_MOTION_MASK) | GDK_SCROLL_MASK) | GDK_TOUCH_MASK);
	gdk_window_set_event_compression(gtk_widget_get_window((GtkWidget *) this->da), false);
	g_signal_connect((GtkWidget *) this->da, "event", G_CALLBACK(on_da_event), this);
	g_signal_connect((GtkWidget *) this->da, "draw", G_CALLBACK(on_da_paint), this);
	myapp_set_document(this, newdoc);
}

void myapp_destruct(myapp *this)
{
	if(this->surf) {
		cairo_surface_destroy(this->surf);
	}

	if(this->doc) {
		image_document_destruct(this->doc);
		free(this->doc);
	}

	this->doc = NULL;
}

void myapp_close_document(myapp *this)
{
	myapp_set_sensitivity_of_doc_controls(this, false);
	if(this->doc) {
		image_document_destruct(this->doc);
		free(this->doc);
	}

	this->doc = NULL;
	myapp_drawframe_noskip(this);
}

_Bool myapp_close_document_with_prompt(myapp *this)
{
	if(!this->doc) {
		return true;
	}

	GtkWidget *__ngg_tmp_50_dlg = gtk_message_dialog_new(this->wnd, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s", "Are you sure you want to close the document?");

	int __ngg_tmp_51_res = gtk_dialog_run(__ngg_tmp_50_dlg);
	gtk_widget_destroy((GtkWidget *) __ngg_tmp_50_dlg);

	if(!(GTK_RESPONSE_YES == (__ngg_tmp_51_res))) {
		return false;
	}

	myapp_close_document(this);

	return true;
}

void myapp_reflect_brush_status(myapp *this)
{
	this->disable_brush_preview = true;
	switch(this->doc->drawparams.brushparams.type) {
	case BRUSHTYPE_CIRCLE1:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_brushtype_circle1, true);
		break;
	}
	case BRUSHTYPE_CIRCLE1NOISE:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_brushtype_circle1noise, true);
		break;
	}
	case BRUSHTYPE_CIRCLE2:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_brushtype_circle2, true);
		break;
	}
	case BRUSHTYPE_RANDSQUARE:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_brushtype_randsquare, true);
		break;
	}
	}

	gtk_toggle_button_set_active((GtkToggleButton *) this->chk_pressure, this->doc->drawparams.brushparams.pressure_size);
	myapp_update_brush_size_view(this);
	entry_slider_set_value_by_code(this->sldr_brush_opacity, this->doc->drawparams.brushparams.opacity);
	entry_slider_set_value_by_code(this->sldr_brush_softness, this->doc->drawparams.brushparams.softness);
	this->disable_brush_preview = false;
	myapp_repreview_brush(this);
}

void myapp_reflect_document_state(myapp *this)
{
	switch(this->doc->drawparams.active_tool) {
	case TOOL_BRUSH:
	{
		if(this->doc->drawparams.erase) {
			gtk_toggle_button_set_active((GtkToggleButton *) this->btn_eraser, true);
		} else {
			gtk_toggle_button_set_active((GtkToggleButton *) this->btn_brush, true);
		}

		break;
	}
	case TOOL_PENCIL:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_pencil, true);
		break;
	}
	case TOOL_EYEDROPPER:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_eyedropper, true);
		break;
	}
	case TOOL_FLOODFILL:
	{
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_floodfill, true);
		break;
	}
	}

	myapp_set_canvas_cursor_for_active_tool(this);
	entry_slider_set_value_by_code(this->sldr_stroke_smoothness, this->doc->stroke_smoothness);
	gtk_toggle_button_set_active((GtkToggleButton *) this->chk_subpixel_when_zoomed, this->doc->subpixel_when_zoomed);
	myapp_reflect_brush_status(this);
	myapp_reflect_color_status(this);
	myapp_update_zoom_controls(this);
	if(this->doc->layer == this->doc->fglayer) {
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_fglayer, true);
	} else {
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_bglayer, true);
	}

	gtk_toggle_button_set_active((GtkToggleButton *) this->chk_bg_visible, this->doc->bgvisible);
	if(this->doc->fgalpha == 1) {
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_fgopacity100, true);
	} else if(this->doc->fgalpha == 0.5) {
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_fgopacity50, true);
	} else if(this->doc->fgalpha == 0) {
		gtk_toggle_button_set_active((GtkToggleButton *) this->btn_fgopacity0, true);
	}

}

void myapp_reflect_color_status(myapp *this)
{
	GdkRGBA __ngg_tmp_52_rgba = (GdkRGBA){ this->doc->drawparams.fgcolor.rgb[0], this->doc->drawparams.fgcolor.rgb[1], this->doc->drawparams.fgcolor.rgb[2], this->doc->drawparams.fgcolor.rgb[3]};
	gtk_color_chooser_set_rgba((GtkColorChooser*) this->btn_color, &__ngg_tmp_52_rgba);
}

void myapp_set_canvas_cursor(myapp *this, const char * name)
{
	struct GdkWindow *gdkwin = gtk_widget_get_window((GtkWidget *) this->da);
	gdk_window_set_cursor(gdkwin, gdk_cursor_new_from_name(gdk_window_get_display(gdkwin), name));
}

void myapp_set_canvas_cursor_for_active_tool(myapp *this)
{
	if(clopts.static_cursor) {
		return ;
	}

	_Bool pick_circle = false;
	switch(this->doc->drawparams.active_tool) {
	case TOOL_BRUSH:
	{
		pick_circle = true;
		break;
	}
	case TOOL_PENCIL:
	{
		pick_circle = true;
		break;
	}
	case TOOL_EYEDROPPER:
	{
		myapp_set_canvas_cursor(this, "default");
		this->curcursor_is_circle = false;
		break;
	}
	case TOOL_FLOODFILL:
	{
		myapp_set_canvas_cursor(this, "pointer");
		this->curcursor_is_circle = false;
		break;
	}
	}

	if(pick_circle) {
		struct GdkWindow *gdkwin = gtk_widget_get_window((GtkWidget *) this->da);
		GdkDisplay *gdkdisp = gdk_window_get_display(gdkwin);
		int maxw = 0;
		int maxh = maxw;
		gdk_display_get_maximal_cursor_size(gdkdisp, &maxw, &maxh);
		int maxsiz = imin2(maxw, maxh);
		if(maxsiz <= 0) {
			fprintf(stderr, "warning: gdk_display_get_maximal_cursor_size() returned <= 0: %d\n", maxsiz);
			unsigned int defsiz = gdk_display_get_default_cursor_size(gdkdisp);
			if(defsiz > 0) {
				maxsiz = defsiz;
			} else {
				fprintf(stderr, "warning: gdk_display_get_default_cursor_size() returned <= 0: %d\n", defsiz);

				return ;
			}

		}

		assert(maxsiz > 0); /* ui.ngg:737 */
		int minsiz = 4;
		int siz = this->doc->drawparams.brushparams.siz;
		if(siz > maxsiz) {
			siz = maxsiz;
		} else if(siz < minsiz) {
			siz = minsiz;
		}

		if(this->curcursor_is_circle && (siz == this->curcursorsiz)) {
			return ;
		} else {
			this->curcursorsiz = siz;
		}


		Image * img = malloc(sizeof(Image));
		if(img == NULL) { perror(NULL); exit(EXIT_FAILURE); }
		image_construct(img, siz, siz, 255);
		image_fill(img, PIXEL_TRANSP);
		int aardius = 1;
		int mid = siz / 2;
		int rad = mid;
		nan_img_draw_circle(img, mid, mid, rad, aardius);
		cairo_surface_t *isurf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, siz, siz);
		copy_image_region_to_cairo_surface(img, isurf, image_get_region(img));
		gdk_window_set_cursor(gdkwin, gdk_cursor_new_from_surface(gdkdisp, isurf, mid, mid));
		cairo_surface_destroy(isurf);
		this->curcursor_is_circle = true;
		if(img) {
			image_destruct(img);
			free(img);
		}

		img = NULL;
	}

}

void myapp_set_document(myapp *this, ImageDocument *newdoc)
{
	myapp_close_document(this);
	this->doc = newdoc;
	assert(this->doc); /* ui.ngg:777 */
	int vpw = imax2(gtk_widget_get_allocated_width((GtkWidget *) this->da), 1);
	int vph = imax2(gtk_widget_get_allocated_height((GtkWidget *) this->da), 1);
	image_document_viewport_set_size(this->doc, vpw, vph);
	image_document_viewport_align_center(this->doc);
	myapp_reflect_document_state(this);
	myapp_drawframe_noskip(this);
	myapp_set_sensitivity_of_doc_controls(this, true);
}

void myapp_set_sensitivity_of_doc_controls(myapp *this, _Bool sensitive)
{
	gtk_widget_set_sensitive((GtkWidget *) this->da, sensitive);
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls1, sensitive);
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls2, sensitive);
	gtk_widget_set_sensitive((GtkWidget *) this->box_controls3, sensitive);
}

void myapp_on_btn_color_set(GtkColorButton *widget, void *user_data)
{
	GdkRGBA __ngg_tmp_53_rgba = (GdkRGBA){};
	gtk_color_chooser_get_rgba((GtkColorChooser*) widget, &__ngg_tmp_53_rgba);

	GdkRGBA rgba = __ngg_tmp_53_rgba;
	((myapp *) user_data)->doc->drawparams.fgcolor = nan_pixel_new(rgba.red, rgba.green, rgba.blue, 1);
}

void myapp_on_quickpalette_colorset(QuickPalette *widget, struct Pixel eventcolor, void *user_data)
{
	if(((myapp *) user_data)->doc) {
		((myapp *) user_data)->doc->drawparams.fgcolor = eventcolor;
		myapp_reflect_color_status(((myapp *) user_data));
	}

}

void myapp_on_btn_bplus_clicked(GtkButton *widget, void *user_data)
{
	((myapp *) user_data)->doc->drawparams.brushparams.siz += 4;
	myapp_update_brush_size_view(((myapp *) user_data));
}

void myapp_on_btn_bminus_clicked(GtkButton *widget, void *user_data)
{
	((myapp *) user_data)->doc->drawparams.brushparams.siz -= 4;
	if(((myapp *) user_data)->doc->drawparams.brushparams.siz < 1) {
		((myapp *) user_data)->doc->drawparams.brushparams.siz = 1;
	}

	myapp_update_brush_size_view(((myapp *) user_data));
}

void myapp_on_btn_tool_x_toggled(GtkCheckButton *widget, void *user_data)
{
	if(!(gtk_toggle_button_get_active((GtkToggleButton *) widget))) {
		return;	}

	enum Tool tool = TOOL_BRUSH;
	if(widget == ((myapp *) user_data)->btn_brush) {
		myapp_pick_tool(((myapp *) user_data), TOOL_BRUSH, false);
	} else if(widget == ((myapp *) user_data)->btn_eraser) {
		myapp_pick_tool(((myapp *) user_data), TOOL_BRUSH, true);
	} else if(widget == ((myapp *) user_data)->btn_pencil) {
		myapp_pick_tool(((myapp *) user_data), TOOL_PENCIL, false);
	} else if(widget == ((myapp *) user_data)->btn_eyedropper) {
		myapp_pick_tool(((myapp *) user_data), TOOL_EYEDROPPER, false);
	} else if(widget == ((myapp *) user_data)->btn_floodfill) {
		myapp_pick_tool(((myapp *) user_data), TOOL_FLOODFILL, false);
	}

}

void myapp_on_btn_brushtype_circlex_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		if(widget == ((myapp *) user_data)->btn_brushtype_circle1) {
			((myapp *) user_data)->doc->drawparams.brushparams.type = BRUSHTYPE_CIRCLE1;
		} else if(widget == ((myapp *) user_data)->btn_brushtype_circle1noise) {
			((myapp *) user_data)->doc->drawparams.brushparams.type = BRUSHTYPE_CIRCLE1NOISE;
		} else {
			((myapp *) user_data)->doc->drawparams.brushparams.type = BRUSHTYPE_CIRCLE2;
		}

		entry_slider_set_sensitive(((myapp *) user_data)->sldr_brush_softness, true);
		myapp_repreview_brush(((myapp *) user_data));
	}

}

void myapp_on_btn_brushtype_randsquare_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		((myapp *) user_data)->doc->drawparams.brushparams.type = BRUSHTYPE_RANDSQUARE;
		entry_slider_set_sensitive(((myapp *) user_data)->sldr_brush_softness, false);
		myapp_repreview_brush(((myapp *) user_data));
	}

}

void myapp_on_mni_fill_bgcolor_activate(GtkMenuItem *widget, void *user_data)
{
	image_document_fill_layer_with_bgcolor(((myapp *) user_data)->doc);
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_mni_fill_fgcolor_activate(GtkMenuItem *widget, void *user_data)
{
	image_document_fill_layer_with_fgcolor(((myapp *) user_data)->doc);
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_mni_tool_x_toggled(GtkMenuItem *widget, void *user_data)
{
	enum Tool tool = TOOL_BRUSH;
	if(widget == ((myapp *) user_data)->mni_tools_brush) {
		myapp_pick_tool(((myapp *) user_data), TOOL_BRUSH, false);
	} else if(widget == ((myapp *) user_data)->mni_tools_eraser) {
		myapp_pick_tool(((myapp *) user_data), TOOL_BRUSH, true);
	} else if(widget == ((myapp *) user_data)->mni_tools_pencil) {
		myapp_pick_tool(((myapp *) user_data), TOOL_PENCIL, false);
	} else if(widget == ((myapp *) user_data)->mni_tools_eyedropper) {
		myapp_pick_tool(((myapp *) user_data), TOOL_EYEDROPPER, false);
	} else if(widget == ((myapp *) user_data)->mni_tools_floodfill) {
		myapp_pick_tool(((myapp *) user_data), TOOL_FLOODFILL, false);
	}

}

void myapp_on_btn_prebrush_clicked(GtkButton *widget, void *user_data)
{
	if(widget == ((myapp *) user_data)->btn_prebrush_ballpoint) {
		((myapp *) user_data)->doc->drawparams.brushparams = prebrush_ballpoint();
	} else if(widget == ((myapp *) user_data)->btn_prebrush_charcoal) {
		((myapp *) user_data)->doc->drawparams.brushparams = prebrush_charcoal();
	} else if(widget == ((myapp *) user_data)->btn_prebrush_round) {
		((myapp *) user_data)->doc->drawparams.brushparams = prebrush_round();
	} else if(widget == ((myapp *) user_data)->btn_prebrush_round_pressure_size) {
		((myapp *) user_data)->doc->drawparams.brushparams = prebrush_round_pressure_size();
	} else if(widget == ((myapp *) user_data)->btn_prebrush_soft_round) {
		((myapp *) user_data)->doc->drawparams.brushparams = prebrush_soft_round();
	} else {
		assert(false); /* ui.ngg:924 */
	}

	myapp_reflect_brush_status(((myapp *) user_data));
}

void myapp_on_sldr_stroke_smoothness_changed(EntrySlider *widget, void *user_data)
{
	((myapp *) user_data)->doc->stroke_smoothness = entry_slider_get_value(widget);
}

void myapp_on_chk_advanced_toggled(GtkCheckButton *widget, void *user_data)
{
	gtk_widget_set_visible((GtkWidget *) ((myapp *) user_data)->sw_controls3, gtk_toggle_button_get_active((GtkToggleButton *) widget));
}

void myapp_on_chk_subpixel_when_zoomed_toggled(GtkCheckButton *widget, void *user_data)
{
	((myapp *) user_data)->doc->subpixel_when_zoomed = gtk_toggle_button_get_active((GtkToggleButton *) widget);
	((myapp *) user_data)->doc->subpixel_when_zoomed = ((myapp *) user_data)->doc->subpixel_when_zoomed;
}

void myapp_on_chk_pressure_toggled(GtkCheckButton *widget, void *user_data)
{
	((myapp *) user_data)->doc->drawparams.brushparams.pressure_size = gtk_toggle_button_get_active((GtkToggleButton *) widget);
	((myapp *) user_data)->doc->drawparams.brushparams.pressure_size = ((myapp *) user_data)->doc->drawparams.brushparams.pressure_size;
	myapp_repreview_brush(((myapp *) user_data));
}

void myapp_on_btn_bglayer_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		((myapp *) user_data)->doc->layer = ((myapp *) user_data)->doc->bglayer;
	}

}

void myapp_on_btn_clear_layer_clicked(GtkButton *widget, void *user_data)
{
	image_document_clear_layer(((myapp *) user_data)->doc);
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_fglayer_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		((myapp *) user_data)->doc->layer = ((myapp *) user_data)->doc->fglayer;
	}

}

void myapp_on_btn_fgopacity100_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		image_document_set_fgalpha(((myapp *) user_data)->doc, 1);
		myapp_drawframe_noskip(((myapp *) user_data));
	}

}

void myapp_on_btn_fgopacity50_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		image_document_set_fgalpha(((myapp *) user_data)->doc, 0.5);
		myapp_drawframe_noskip(((myapp *) user_data));
	}

}

void myapp_on_btn_fgopacity0_toggled(GtkCheckButton *widget, void *user_data)
{
	if(gtk_toggle_button_get_active((GtkToggleButton *) widget)) {
		image_document_set_fgalpha(((myapp *) user_data)->doc, 0);
		myapp_drawframe_noskip(((myapp *) user_data));
	}

}

void myapp_on_btn_export_doc_clicked(GtkButton *widget, void *user_data)
{
	char * fnam = myapp_prompt_for_export_filename(((myapp *) user_data));
	if(!fnam) {
		return;	}

	image_document_export_document(((myapp *) user_data)->doc, fnam, false);
	if(fnam) {
		free(fnam);
	}

	fnam = NULL;
}

void myapp_on_btn_export_layer_clicked(GtkButton *widget, void *user_data)
{
	char * fnam = myapp_prompt_for_export_filename(((myapp *) user_data));
	if(!fnam) {
		return;	}

	image_document_export_layer(((myapp *) user_data)->doc, fnam, false);
	if(fnam) {
		free(fnam);
	}

	fnam = NULL;
}

void myapp_on_btn_undo_clicked(GtkButton *widget, void *user_data)
{
	image_document_undo(((myapp *) user_data)->doc);
	myapp_update_history_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_redo_clicked(GtkButton *widget, void *user_data)
{
	image_document_redo(((myapp *) user_data)->doc);
	myapp_update_history_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_center_clicked(GtkButton *widget, void *user_data)
{
	image_document_viewport_align_center(((myapp *) user_data)->doc);
	myapp_update_scroll_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_zoom100_clicked(GtkButton *widget, void *user_data)
{
	image_document_viewport_zoom_at_center(((myapp *) user_data)->doc, 1);
	myapp_update_zoom_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_zoomin_clicked(GtkButton *widget, void *user_data)
{
	image_document_viewport_zoom_at_center(((myapp *) user_data)->doc, ((myapp *) user_data)->doc->vpzoom + ZOOMSTEP);
	myapp_update_zoom_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_btn_zoomout_clicked(GtkButton *widget, void *user_data)
{
	double zoom = ((myapp *) user_data)->doc->vpzoom - ZOOMSTEP;
	assert(zoom >= ZOOMSTEP); /* ui.ngg:1040 */
	image_document_viewport_zoom_at_center(((myapp *) user_data)->doc, zoom);
	myapp_update_zoom_controls(((myapp *) user_data));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_chk_bg_visible_toggled(GtkCheckButton *widget, void *user_data)
{
	image_document_set_bgvisible(((myapp *) user_data)->doc, gtk_toggle_button_get_active((GtkToggleButton *) widget));
	myapp_drawframe_noskip(((myapp *) user_data));
}

char * myapp_prompt_for_export_filename(myapp *this)
{
	GtkWidget *__ngg_tmp_54_dlg = gtk_file_chooser_dialog_new("Save File", this->wnd, GTK_FILE_CHOOSER_ACTION_SAVE, "Cancel", GTK_RESPONSE_CANCEL, "Save", GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation((GtkFileChooser*) __ngg_tmp_54_dlg, true);

	GtkDialog *dlg = (GtkDialog*) __ngg_tmp_54_dlg;
	GtkFileFilter *filter = gtk_file_filter_new();
	gtk_file_filter_add_mime_type(filter, "image/png");
	gtk_file_filter_set_name(filter, "*.png");
	gtk_file_chooser_add_filter((GtkFileChooser*) dlg, filter);

	int __ngg_tmp_55_res = gtk_dialog_run(dlg);
	const char * __ngg_tmp_55_fnam = NULL;
	if(__ngg_tmp_55_res == GTK_RESPONSE_ACCEPT) {
		__ngg_tmp_55_fnam = gtk_file_chooser_get_filename((GtkFileChooser*) dlg);
	}

	gtk_widget_destroy((GtkWidget *) dlg);

	return __ngg_tmp_55_fnam;
}

void myapp_on_da_sizeallocate(GtkDrawingArea *widget, GdkRectangle *nggevent_allocation, void *user_data)
{
	if(!(gtk_widget_get_mapped((GtkWidget *) widget))) {
		return;	}

	myapp_reserve_cairo_surface(((myapp *) user_data), nggevent_allocation->width, nggevent_allocation->height);
	if(((myapp *) user_data)->doc) {
		image_document_viewport_set_size(((myapp *) user_data)->doc, nggevent_allocation->width, nggevent_allocation->height);
		myapp_update_scroll_controls(((myapp *) user_data));
	}

	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_hscroll_valuechanged(GtkRange *widget, void *user_data)
{
	if(((myapp *) user_data)->was_autoscroll) {
		return;	}

	image_document_viewport_set_offsetx(((myapp *) user_data)->doc, 0 - (gtk_range_get_value(widget)));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_on_sldr_brush_opacity_changed(EntrySlider *widget, void *user_data)
{
	((myapp *) user_data)->doc->drawparams.brushparams.opacity = entry_slider_get_value(widget);
	myapp_repreview_brush(((myapp *) user_data));
}

void myapp_on_sldr_brush_softness_changed(EntrySlider *widget, void *user_data)
{
	((myapp *) user_data)->doc->drawparams.brushparams.softness = entry_slider_get_value(widget);
	myapp_repreview_brush(((myapp *) user_data));
}

void myapp_on_vscroll_valuechanged(GtkRange *widget, void *user_data)
{
	if(((myapp *) user_data)->was_autoscroll) {
		return;	}

	image_document_viewport_set_offsety(((myapp *) user_data)->doc, 0 - (gtk_range_get_value(widget)));
	myapp_drawframe_noskip(((myapp *) user_data));
}

void myapp_pick_tool(myapp *this, enum Tool tool, _Bool erase)
{
	switch(tool) {
	case TOOL_BRUSH:
	{
		this->doc->drawparams.active_tool = TOOL_BRUSH;
		this->doc->drawparams.erase = erase;
		break;
	}
	case TOOL_PENCIL:
	{
		this->doc->drawparams.active_tool = TOOL_PENCIL;
		this->doc->drawparams.erase = false;
		break;
	}
	case TOOL_EYEDROPPER:
	{
		this->doc->drawparams.active_tool = TOOL_EYEDROPPER;
		break;
	}
	case TOOL_FLOODFILL:
	{
		this->doc->drawparams.active_tool = TOOL_FLOODFILL;
		break;
	}
	}

	myapp_set_canvas_cursor_for_active_tool(this);
}

void myapp_reserve_cairo_surface(myapp *this, int w, int h)
{
	assert(w > 0); /* ui.ngg:1152 */
	assert(h > 0); /* ui.ngg:1153 */
	if(this->surf) {
		if(((cairo_image_surface_get_width(this->surf)) >= w) && ((cairo_image_surface_get_height(this->surf)) >= h)) {
			return ;
		}

		cairo_surface_destroy(this->surf);
	}

	this->surf = gdk_window_create_similar_image_surface(gtk_widget_get_window((GtkWidget *) this->da), CAIRO_FORMAT_RGB24, w, h, 0);
}

void myapp_update_brush_size_view(myapp *this)
{
	const size_t _ngg_tmp_56 = 16;
	const size_t _ngg_tmp_57 = _ngg_tmp_56 + 1;
	size_t _ngg_tmp_58 = 0;
	char * _ngg_tmp_59 = malloc(_ngg_tmp_57);
	if(!_ngg_tmp_59) { perror(NULL); exit(EXIT_FAILURE); }
	snprintf(_ngg_tmp_59 + _ngg_tmp_58, 16, "%d", this->doc->drawparams.brushparams.siz);
	_ngg_tmp_58 = strlen(_ngg_tmp_59);
	_ngg_tmp_59[_ngg_tmp_58] = '\0';
	char * _ngg_tmp_7 = _ngg_tmp_59;
	gtk_label_set_label(this->lbl_brush_size, _ngg_tmp_7);
	myapp_repreview_brush(this);
	myapp_set_canvas_cursor_for_active_tool(this);
	if(_ngg_tmp_7) {
		free(_ngg_tmp_7);
	}

	_ngg_tmp_7 = NULL;
}

void myapp_update_history_controls(myapp *this)
{
	gtk_widget_set_sensitive((GtkWidget *) this->btn_undo, image_document_can_undo(this->doc));
	gtk_widget_set_sensitive((GtkWidget *) this->btn_redo, image_document_can_redo(this->doc));
}

void myapp_update_scroll_controls(myapp *this)
{
	if(!this->doc->preview) {
		return ;
	}

	this->was_autoscroll = true;
	double curx = 0 - this->doc->vpx;
	double cury = 0 - this->doc->vpy;
	int psx = this->doc->preview->width;
	int psy = this->doc->preview->height;
	double xmax = imax2(this->doc->vpzoom * this->doc->composite->width, curx + psx);
	double ymax = imax2(this->doc->vpzoom * this->doc->composite->height, cury + psy);
	int xmin = imin2(curx, 0);
	int ymin = imin2(cury, 0);
	assert(xmin <= xmax); /* ui.ngg:1218 */
	assert(ymin <= ymax); /* ui.ngg:1219 */
	assert(curx >= xmin && curx <= xmax); /* ui.ngg:1220 */
	assert(cury >= ymin && cury <= ymax); /* ui.ngg:1221 */
	gtk_range_set_range((GtkRange *) this->hscroll, xmin, xmax);
	gtk_range_set_range((GtkRange *) this->vscroll, ymin, ymax);
	gtk_adjustment_set_page_size(gtk_range_get_adjustment((GtkRange *) this->hscroll), psx);
	gtk_adjustment_set_page_size(gtk_range_get_adjustment((GtkRange *) this->vscroll), psy);
	gtk_range_set_value((GtkRange *) this->hscroll, curx);
	gtk_range_set_value((GtkRange *) this->vscroll, cury);
	this->was_autoscroll = false;
}

void myapp_update_zoom_controls(myapp *this)
{
	gtk_widget_set_sensitive((GtkWidget *) this->btn_zoomin, true);
	gtk_widget_set_sensitive((GtkWidget *) this->btn_zoomout, this->doc->vpzoom >= (2 * ZOOMSTEP));
	const size_t _ngg_tmp_60 = 16;
	const size_t _ngg_tmp_61 = strlen("%");
	const size_t _ngg_tmp_62 = _ngg_tmp_60 + _ngg_tmp_61 + 1;
	size_t _ngg_tmp_63 = 0;
	char * _ngg_tmp_64 = malloc(_ngg_tmp_62);
	if(!_ngg_tmp_64) { perror(NULL); exit(EXIT_FAILURE); }
	snprintf(_ngg_tmp_64 + _ngg_tmp_63, 16, "%d", (int) (100 * this->doc->vpzoom));
	_ngg_tmp_63 = strlen(_ngg_tmp_64);
	memcpy(_ngg_tmp_64 + _ngg_tmp_63, "%", _ngg_tmp_61);
	_ngg_tmp_63 += _ngg_tmp_61;
	_ngg_tmp_64[_ngg_tmp_63] = '\0';
	char * _ngg_tmp_8 = _ngg_tmp_64;
	gtk_label_set_label(this->lbl_zoom, _ngg_tmp_8);
	myapp_update_scroll_controls(this);
	if(_ngg_tmp_8) {
		free(_ngg_tmp_8);
	}

	_ngg_tmp_8 = NULL;
}

void myapp_process_da_event(myapp *this, int event_type, int x, int y, double pressure, int scroll_direction)
{
	switch(event_type) {
	case GDK_BUTTON_PRESS:
	case GDK_TOUCH_BEGIN:
	{
		if(this->doc->drawparams.active_tool == TOOL_EYEDROPPER || this->doc->drawparams.active_tool == TOOL_FLOODFILL) {
			return ;
		}

		image_document_pen_down(this->doc, x, y, pressure);
		myapp_drawframe(this);
		break;
	}
	case GDK_BUTTON_RELEASE:
	case GDK_TOUCH_CANCEL:
	case GDK_TOUCH_END:
	{
		if(this->doc->drawparams.active_tool == TOOL_EYEDROPPER) {
			image_document_dip_eyedropper(this->doc, x, y);
			myapp_reflect_color_status(this);

			return ;
		} else if(this->doc->drawparams.active_tool == TOOL_FLOODFILL) {
			image_document_floodfill(this->doc, x, y);
			myapp_drawframe_noskip(this);

			return ;
		}

		image_document_pen_up(this->doc, x, y, pressure);
		myapp_drawframe_noskip(this);
		myapp_update_history_controls(this);
		break;
	}
	case GDK_MOTION_NOTIFY:
	case GDK_TOUCH_UPDATE:
	{
		if(this->doc->pen_is_down) {
			image_document_pen_stroke(this->doc, x, y, pressure);
			myapp_drawframe(this);
		}

		break;
	}
	case GDK_SCROLL:
	{
		switch(scroll_direction) {
		case GDK_SCROLL_UP:
		{
			double zoom = this->doc->vpzoom + ZOOMSTEP;
			image_document_viewport_zoom_at(this->doc, x, y, zoom);
			break;
		}
		case GDK_SCROLL_DOWN:
		{
			if(this->doc->vpzoom <= ZOOMSTEP) {
				return ;
			}

			double zoom = this->doc->vpzoom - ZOOMSTEP;
			image_document_viewport_zoom_at(this->doc, x, y, zoom);
			break;
		}
		}

		myapp_update_zoom_controls(this);
		myapp_drawframe_noskip(this);
		break;
	}
	default:
	{
		break;
	}
	}

}

struct Rectangle myapp_copy_preview_to_cairo_surface(myapp *this)
{
	assert(this->doc); /* ui.ngg:1336 */
	struct Rectangle dirtyrect = image_document_update_preview(this->doc);
	copy_image_region_to_cairo_surface(this->doc->preview, this->surf, dirtyrect);

	return dirtyrect;
}

void myapp_clear_cairo_surface(myapp *this)
{
	assert(this->surf); /* ui.ngg:1350 */
	int x = 0;
	int y = x;
	int stride = cairo_image_surface_get_stride(this->surf);
	unsigned char *ibuf = cairo_image_surface_get_data(this->surf);
	size_t rowpos = 0;
	for(y = 0; y < cairo_image_surface_get_height(this->surf); ++y) {
		size_t pixpos = rowpos;
		for(x = 0; x < cairo_image_surface_get_width(this->surf); ++x) {
			nan_copy_pixel_to_rgb24(PIXEL_VPBG, ibuf + pixpos);
			pixpos = pixpos + 4;
		}

		rowpos = rowpos + stride;
	}

	cairo_surface_mark_dirty(this->surf);
}

void myapp_drawframe(myapp *this)
{
	if(gtk_events_pending()) {
		return ;
	}

	myapp_drawframe_noskip(this);
}

void myapp_drawframe_noskip(myapp *this)
{
	if(this->doc) {
		struct Rectangle dirtyrect = myapp_copy_preview_to_cairo_surface(this);
		if(!(region_is_zero(dirtyrect))) {
			gtk_widget_queue_draw_area((GtkWidget *) this->da, dirtyrect.x, dirtyrect.y, dirtyrect.w, dirtyrect.h);
		}

	} else {
		myapp_clear_cairo_surface(this);
		gtk_widget_queue_draw((GtkWidget *) this->da);
	}

}

void myapp_on_btn_close_doc_clicked(GtkButton *widget, void *user_data)
{
	myapp_close_document_with_prompt(((myapp *) user_data));
}

void myapp_on_btn_help_about_clicked(GtkButton *widget, void *user_data)
{
	GtkAboutDialog *dlg = (GtkAboutDialog*) (gtk_about_dialog_new());
	gtk_about_dialog_set_program_name(dlg, PROGRAM_NAME);
	gtk_about_dialog_set_comments(dlg, "Just draw.");
	const char * _ngg_tmp_65[2];
	_ngg_tmp_65[0] = "Nandakumar Edamana";
	_ngg_tmp_65[1] = NULL;
	gtk_about_dialog_set_authors(dlg, _ngg_tmp_65);
	gtk_about_dialog_set_copyright(dlg, "Copyright (C) 2021, 2023 Nandakumar Edamana <contact@nandakumar.co.in>");
	gtk_about_dialog_set_version(dlg, VERSION);
	gtk_about_dialog_set_website(dlg, "https://nandakumar.co.in/software/vara/");
	gtk_window_set_transient_for((GtkWindow *) dlg, ((myapp *) user_data)->wnd);
	int __ngg_tmp_66_res = gtk_dialog_run(dlg);
	gtk_widget_destroy((GtkWidget *) dlg);

	__ngg_tmp_66_res;
}

void myapp_on_btn_new_doc_clicked(GtkButton *widget, void *user_data)
{
	if(!(myapp_close_document_with_prompt(((myapp *) user_data)))) {
		return;	}


	NewDocDialog * ndlg = malloc(sizeof(NewDocDialog));
	if(ndlg == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	new_doc_dialog_construct(ndlg, ((myapp *) user_data)->wnd, MAXWIDTH);
	struct _ngg_tuple_run _ngg_tmp_4 = new_doc_dialog_run(ndlg);
	int h = _ngg_tmp_4.m2;
	int w = _ngg_tmp_4.m1;
	_Bool ok = _ngg_tmp_4.m0;
	if(!ok) {
		return;	}


	ImageDocument * _ngg_tmp_67 = malloc(sizeof(ImageDocument));
	if(_ngg_tmp_67 == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_document_construct(_ngg_tmp_67, w, h);
	myapp_set_document(((myapp *) user_data), _ngg_tmp_67);
	if(ndlg) {
		new_doc_dialog_destruct(ndlg);
		free(ndlg);
	}

	ndlg = NULL;
}

void myapp_repreview_brush(myapp *this)
{
	if(this->disable_brush_preview) {
		return ;
	}

	draw_brush_preview(this->iv_brushpreview->img, this->doc->drawparams, false);
	image_view_queue_draw(this->iv_brushpreview);
}

void on_da_event(GtkWidget *w, GdkEvent *event, myapp *app)
{
	struct _ngg_tuple_ngg_coords_from_event _ngg_tmp_3 = ngg_coords_from_event(event);
	double y = _ngg_tmp_3.m1;
	double x = _ngg_tmp_3.m0;
	double pressure = 0.0;
	bool present = gdk_event_get_axis(event, GDK_AXIS_PRESSURE, &pressure);
	if(!present) pressure = 1;
	int dir = 0;
	if(event->type == GDK_SCROLL) {
		gdk_event_get_scroll_direction(event, &dir);
	}

	myapp_process_da_event(app, event->type, x, y, pressure, dir);
}

void on_da_paint(GtkWidget *w, cairo_t *cr, myapp *app)
{
	cairo_set_source_surface(cr, app->surf, 0, 0);
	cairo_paint(cr);
}

_Bool on_wnd_deleteevent(GtkWidget *wgt, GdkEvent *event, myapp *app)
{
	if(!(myapp_close_document_with_prompt(app))) {
		return true;
	}

	if(app) {
		myapp_destruct(app);
		free(app);
	}

	app = NULL;
	gui_quit();

	return false;
}

void draw_brush_preview(Image *img, struct DrawingParams dpcopy, _Bool dot_only)
{
	image_fill(img, PIXEL_WHITE);
	dpcopy.fgcolor = PIXEL_BLACK;
	dpcopy.oldpressure = 1;
	double radius = dpcopy.brushparams.siz / 2.0;
	double ymid = img->height / 2.0;
	if(dot_only) {
		double xmid = img->width / 2.0;
		dpcopy.pressure = 1;
		image_line(img, xmid, ymid, xmid, ymid, dpcopy);
	} else {
		int padding = 5;
		dpcopy.pressure = 0;
		image_line(img, padding + radius, ymid, (img->width - radius) - padding, ymid, dpcopy);
	}

}

GtkWidget* brushpreview_widget_new(struct BrushParams brushparams)
{

	ImageView * iv = malloc(sizeof(ImageView));
	if(iv == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_view_construct(iv);

	Image * img = malloc(sizeof(Image));
	if(img == NULL) { perror(NULL); exit(EXIT_FAILURE); }
	image_construct(img, 36, 24, 255);
	struct DrawingParams dp = default_drawparams();
	dp.brushparams = brushparams;
	dp.active_tool = TOOL_BRUSH;
	draw_brush_preview(img, dp, false);
	image_view_set_image(iv, img);

	return image_view_get_native_widget(iv);
}

CommandLineOptions command_line_options_default()
{
	CommandLineOptions s;
	s.static_cursor = false;

	return s;
}

